==> peekabot-0.6.0.tar.bz2: peekabot-0.6.0/src/actions/RotateCamera.cc <==
/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "RotateCamera.hh"
#include "../Matrix.hh"
#include "../Vector.hh"


#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#  include "../CameraObject.hh"
#endif


using namespace peekabot;


RotateCamera::RotateCamera() throw()
{
}


RotateCamera::RotateCamera(
    ObjectID target, real_unit delta_x, real_unit delta_y) throw()
    : m_target(target),
      m_delta_x(delta_x), m_delta_y(delta_y)
{
}


Action *RotateCamera::clone() const
{
    return new RotateCamera(m_target, m_delta_x, m_delta_y);
}


void RotateCamera::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    CameraObject *ptr = dynamic_cast<CameraObject *>(
        context->get_object(m_target));

    if( !ptr )
        throw std::runtime_error(
            "Object not found, or of non-camera type.");
    
    Matrix4ru M = ptr->get_mtow();
    Vector4ru pos = M(3);
    M(3) = Vector4ru(0,0,0,1);

    M *= Matrix4ru::rotate(
        M(1), -m_delta_y*ptr->get_fov());

    M *= Matrix4ru::rotate_z(
        -m_delta_x*ptr->get_fov());

    M(3) = pos;

    ptr->set_transformation(
        ptr->get_parent_mtow().inverse() * M);
#endif
}


void RotateCamera::save(SerializationInterface &ar) const
{
    ar << m_target << m_delta_x << m_delta_y;
}

void RotateCamera::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_delta_x >> m_delta_y;
}

==> peekabot-0.7.2.tar.bz2: peekabot-0.7.2/src/actions/RotateCamera.cc <==
/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "RotateCamera.hh"
#include "../Matrix.hh"
#include "../Vector.hh"


#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#  include "../CameraObject.hh"
#endif


using namespace peekabot;


RotateCamera::RotateCamera() throw()
{
}


RotateCamera::RotateCamera(
    ObjectID target, real_unit delta_x, real_unit delta_y) throw()
    : m_target(target),
      m_delta_x(delta_x), m_delta_y(delta_y)
{
}


Action *RotateCamera::clone() const
{
    return new RotateCamera(m_target, m_delta_x, m_delta_y);
}


void RotateCamera::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    CameraObject *ptr = dynamic_cast<CameraObject *>(
        context->get_object(m_target));

    if( !ptr )
        throw std::runtime_error(
            "Object not found, or of non-camera type.");
    
    Matrix4ru M = ptr->get_mtow();
    Vector4ru pos = M(3);
    M(3) = Vector4ru(0,0,0,1);

    M *= Matrix4ru::rotate(
        M(1), m_delta_y*ptr->get_fov());

    M *= Matrix4ru::rotate_z(
        -m_delta_x*ptr->get_fov());

    M(3) = pos;

    ptr->set_transformation(
        ptr->get_parent_mtow().inverse() * M);
#endif
}


void RotateCamera::save(SerializationInterface &ar) const
{
    ar << m_target << m_delta_x << m_delta_y;
}

void RotateCamera::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_delta_x >> m_delta_y;
}
